#!/bin/bash
################################################################################
#                                 demo.sh                                      #
#                                                                              #
# Automated demo script for the All Things Open conference in Raleigh NC, USA  #
# October 2023.                                                                #
#                                                                              #
# Prevents fumbling on the CLI during the presentation. Besides, who wants to  #
# see a bunch of presentation slides when the presentation is about BASH?!     #
#                                                                              #
# NOTE: Can only go forward. Cannot go backwards or enter a slide number.      #
#       Hopefully I can add that capability later but a bit of preplanning     #
#       for that shows it will be a complex task.                              #
#                                                                              #
#------------------------------------------------------------------------------#
#                                                                              #
# You will need to install the following packages for this program.            #
# asciiquarium                                                                 #
# banner                                                                       #
# boxes                                                                        #
# cowsay                                                                       #
# figlet                                                                       #
# lolcat                                                                       #
# neofetch                                                                     #
# sl                                                                           #
#                                                                              #
#                                                                              #
#                                                                              #
#------------------------------------------------------------------------------#
# Change History                                                               #
#------------------------------------------------------------------------------#
#   Date         Who                     Description                           #
#------------------------------------------------------------------------------#
# 02/14/2020  David Both    Original code. Demo automation script for the      #
#                           Open Source 101 conference in Columbia SC          #
# 12/01/2022  David Both    Revised for OLF.                                   #
# 12/02/2022  David Both    Further revised for OLF. Mostly shortened and      #
#                           made more concise.                                 #
# 08/31/2023  David Both    Further revised for ATO 2023.                      #
# 09/12/2023  David Both    Add Extra variable to allow showing Extra stuff    #
#                           for some audiences.                                #
# 09/13/2023  David Both    Add FigFont variable to set font for Figlet.       #
# 10/03/2023  David Both    Add -s option to skip slides not usually used. The #
#                           default is that this is on and using -s adds the   #
#                           skipped slides.                                    #
#                                                                              #
################################################################################
################################################################################
#                                                                              #
#  Copyright (C) 2020. 2023 David Both                                         #
#  LinuxGeek46@both.org                                                        #
#  http://www.both.org                                                         #
#                                                                              #
#  This program is free software; you can redistribute it and/or modify        #
#  it under the terms of the GNU General Public License as published by        #
#  the Free Software Foundation; either version 2 of the License, or           #
#  (at your option) any later version.                                         #
#                                                                              #
#  This program is distributed in the hope that it will be useful,             #
#  but WITHOUT ANY WARRANTY; without even the implied warranty of              #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               #
#  GNU General Public License for more details.                                #
#                                                                              #
#  You should have received a copy of the GNU General Public License           #
#  along with this program; if not, write to the Free Software                 #
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   #
#                                                                              #
################################################################################
################################################################################

################################################################################
# NOTE: This code violates the Unix Programming principle:                     #
#   "Keep the data separate from policy."                                      #
################################################################################

################################################################################
# Help                                                                         #
################################################################################
Help()
{
   # Display Help
   echo "Configuring and Using Bash"
   echo
   echo "Syntax: demo.sh [b|h|g|s|t|v|V|x]"
   echo "options:"
   echo "b     Show book covers."
   echo "g     Print the GPL license notification."
   echo "h     Print this Help."
   echo "s     Print the stuff we normally skip. Keeps the "
   echo "      presentation shorter if we don't use this."
   echo "t     Test mode. Don't show images or videos."
   echo "V     Print software version and exit."
   echo "x     Display extra slides for Doug McIlroy and Pipes."
   echo
}

################################################################################
# Print the GPL license header                                                 #
################################################################################
gpl()
{
   echo
   echo "################################################################################"
   echo "#  Copyright (C) 2020, 2023, David Both                                        #"
   echo "#  LinuxGeek46@both.org                                                        #"
   echo "#  http://www.both.org                                                         #"
   echo "#                                                                              #"
   echo "#  This program is free software; you can redistribute it and/or modify        #"
   echo "#  it under the terms of the GNU General Public License as published by        #"
   echo "#  the Free Software Foundation; either version 2 of the License, or           #"
   echo "#  (at your option) any later version.                                         #"
   echo "#                                                                              #"
   echo "#  This program is distributed in the hope that it will be useful,             #"
   echo "#  but WITHOUT ANY WARRANTY; without even the implied warranty of              #"
   echo "#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               #"
   echo "#  GNU General Public License for more details.                                #"
   echo "#                                                                              #"
   echo "#  You should have received a copy of the GNU General Public License           #"
   echo "#  along with this program; if not, write to the Free Software                 #"
   echo "#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   #"
   echo "################################################################################"
   echo
}

################################################################################
# Gets simple (Y)es (N)o (Q)uit response from user. Loops until                #
# one of those responses is detected.                                          #
#                                                                              #
# This code is not used in this program but it provides an excellent example   #
# of using a while loop with if-then-elif to accept and examine input.         #
################################################################################
ynq()
{
   # loop until we get a good answer and break out
   while [ "$OK" = 0 ]
   do
      # Print the message
      echo -n "$message (ynq) "
      # Now get some input
      read -r input
      # Test the input
      if [ "$input" = "yes" ] || [ "$input" = "y" ]
      then
         response="y"
         OK=1
      elif [ "$input" = "no" ] || [ "$input" = "n" ]
      then
         response="n"
         OK=1
      elif [ "$input" = "help" ] || [ "$input" = "h" ]
      then
         help
      elif [ "$input" = "q" ] || [ "$input" = "quit" ]
      then
         Quit
      else
         # Invalid input
         echo "INPUT ERROR: Must be y or n or q in lowercase. Please try again."
      fi
   done
}

################################################################################
# Quit nicely with messages as appropriate                                     #
################################################################################
Quit()
{
   if [ $verbose = 1 ]
      then
      if [ $error = 0 ]
         then
         echo "Program terminated normally"
      else
         echo "Program terminated with error ID $ErrorMsg";
      fi
   fi
   exit $error
}

################################################################################
# Display verbose messages in a common format                                  #
################################################################################
PrintMsg()
{
   if  [ $verbose = 1 ] && [ -n "$Msg" ]
   then
      echo "########## $Msg ##########"
      # Set the message to null
      Msg=""
   fi
}


################################################################################
# Display graphics in okular                                                   #
################################################################################
DispGraphic()
{
   # Do this only if Test mode is off
   if [ $Test == 0 ]
   then
      okular $Pic > /dev/null 2>&1 &
      read -r
   fi
}

################################################################################
# Display video in vlc                                                         #
################################################################################
PlayVideo()
{
   # Do this only if Test mode is off
   if [ $Test == 0 ]
   then
      vlc --no-audio -v 0 $Video  > /dev/null 2>&1
      read -r
   fi
}

################################################################################
# Display slides in a box.                                                     #
################################################################################
Slide()
{
   # Clear the display
   clear
   # Display the message as a slide in the defined box
   # $Box must be predefined or it will use the default box command type.
   echo -e "$Msg" | boxes -a $Format -s $Width  -d $Box -p $Pad
   read -r
}

################################################################################
# Display slides in a box with rainbow colors.                                 #
################################################################################
lolSlide()
{
   # Clear the display
   clear
   # Display the message as a slide in the defined box
   # $Box must be predefined or it will use the default box command type.
   figlet -c -f $FigFont $Msg | boxes -d $Box | lolcat
   read -r
}

################################################################################
# Display code as is with indents and all                                      #
################################################################################
Code()
{
   # Clear the screen
   clear
   # Use $Msg variable but keeps basic text formatting
   cat <<EOF
################################################################################
$Msg
################################################################################
EOF
   read -r
}

################################################################################
# Display command, do it amd then wait for Enter key to be pressed for the     #
# next command.                                                                #
################################################################################
DoIt()
{
   # Display the command as if on the Bash CLI
   printf "[$USER $(basename $PWD)]\$ $Command"
   # Wait for Enter key
   read -r
   # Execute the command only if Test mode is off
   if [ $Test == 0 ]
   then
      # Perform the command
      eval $Command 
      Command=''
      printf "[$USER `basename $PWD`]\$ $Command"
      # Wait for Enter key
      read -r
   fi
}

################################################################################
################################################################################
# Main program                                                                 #
################################################################################
################################################################################
# Set initial variables. Keeping this all in one place makes it easier to      #
# keep track of them.                                                          #
################################################################################
badoption=0
Books=0
Box="shell"
error=0
Extra=0
FigFont=script
Files=0
Format="c"
Host=`echo $HOSTNAME | awk -F . '{print $1}'`
Pad="a1l3"
PresentationPWD=`pwd`
Msg=" "
MyVar=""
Pic=""
RC=0
Release=0
Skip=1
Test=0
verbose=0
version=01.10
Width=80


################################################################################
# Process the input options. Add options as needed.                            #
################################################################################
# Get the options
while getopts ":bghstvVx" option; do
   case $option in
      b) # display book info
         Books=1;;
      g) # display GPL
         gpl
         Quit;;
      h) # display Help
         Help
         Quit;;
      s) # Set skip mode to OFF!!!
         Skip=0;;
      t) # Set test mode
         Test=1;;
      v) # Set verbose mode
         verbose=1;;
      V) # Display software version
         echo "Version = $version"
         Quit;;
      x) # Set Extra mode
         Extra=1;;
     \?) # incorrect option
         badoption=1;;
   esac
done

if [ $badoption = 1 ]
then
   echo "ERROR: Invalid option"
   Help
   verbose=1
   error=1
   ErrorMsg=" 10T"
   Quit $error
fi


################################################################################
################################################################################
# The main body of your program goes here.
################################################################################
################################################################################
# clear the screen
clear


################################################################################
# Use this to do page size setup before starting the rest of the presentation  #
# using Ctrl-+ and Ctrl--                                                      #
################################################################################
Box="shell"
Format="c"
FigFont="banner"
Msg="Configuring and using Bash Setup"
lolSlide

# run this as people are entering the room
if [ $Test == 0 ]
then
   asciiquarium
fi

Box="shell"
Format="c"
FigFont="banner"
Msg="Configuring and Using Bash"
lolSlide



################################################################################
# Normally we skip this bit
################################################################################
if [ $Skip == 0 ]
then
   ################################################################################
   # Show the code
   ################################################################################
   Msg=" The Code
   ====================================================
   
   figlet -c -f \$FigFont \$Msg | boxes -d \$Box | lolcat"
   Slide
fi

Box=ian_jones
Format="c"
Msg="David Both
====================================================
RHCE, SCSA

More than 150 published articles
Red Hat On-line Publications
Opensource.com -- Enable SysAdmin -- Architect
Linux Journal, Linux Magazine, OS/2 Magazine

Five books with Apress

http://www.both.org  LinuxGeek46@both.org

Mastodon @LinuxGeek46@linuxrocks.online"
Slide

Box=shell
Format="l"
Msg="My Books
====================================================

o The Linux Philosophy for SysAdmins

o Using and Administering Linux: Zero to SysAdmin
-    Three Volume Self Study Course
-    2nd Edition

o Linux for Small Business Owners
-    Co-author Cyndi Bulka
"
Slide

################################################################################
# This would display the graphic of my books but
# I skip this most of the time.
################################################################################
if [ $Books == 1 ]
then
   Pic="2ndEd-Covers.png"
   DispGraphic > /dev/null
fi



Box=shell
Format="l"
Msg=" Opensource.com -- Enable SysAdmin -- Architect
====================================================

o No longer actively adding new content

o Red Hat indicates existing content will remain available indefinitely
"
Slide


################################################################################
# Announce the move to Opensource.net
################################################################################
Box=html
Format="c"
Msg="NEW HOME
For Opensource.com Community and Writers
====================================================

Opensource.net
https://opensource.net/

Supported by and Hosted at OSI
Open Source Initiative
https://opensource.org/" 
Slide

Box="shell"
Format="l"
FigFont="big"
Msg="NEVER USED WINDOWS
AS PRIMARY OS"
lolSlide

Box="shell"
Format="l"
Msg=" My Choices For My Personal Systems
====================================================

*  DOS

*  DOS + TopView

*  OS/2

*  Linux

"
Slide


Box="shell"
Format="c"
#FigFont="big"
FigFont="banner"
Msg="Why Not Use 
LibreOffice Impress?"
lolSlide

Pic="bash-logo-by-vd.png"
DispGraphic > /dev/null


Box="shell"
Format="l"
FigFont="banner"
Msg="Topics
====================================================

*  The Linux Philosophy
*  Terminology and Architecture of the Command Line
*  Terminals and Terminal Emulators
*  What Everything is a File means on the CLI
*  Data Streams
*  Standard I/O
*  Shell Types
-    Login, Non-login, nologin
*  Shell Configuration Files
*  How to Determine Config File Execution Sequence
*  Efficiency on the CLI
*  Expansions
*  Program Flow Control
*  sudo
*  Licensing 
"
Slide


################################################################################
# Normally we skip this bit
################################################################################
if [ $Skip == 0 ]
then
   Box="shell"
   Format="l"
   Msg=" You Might be a SysAdmin if you ...
   ====================================================
   
   o   Like to take things apart to see how they work
   
   o   Use an open source license for your code.
   
   o   Understand the difference between free as in beer and free
   .   as in speech when applied to software.
   
   o   Have ever repaired or upgraded your own computer.
   
   o   Leave the covers off your computer because you replace
   .   hardware components frequently."
   Slide
   
   
   Box="shell"
   Format="c"
   Msg="The REAL Reason
   ====================================================
   
   Computers are easy – people are hard
   
   – Bridget Kromhout
   
   "
   Slide

fi

Box="parchment"
Format="c"
Msg=" The Linux Truth
====================================================

Unix was not designed to stop its users 
from doing stupid things, as that would 
also stop them from doing clever things.

-- Doug Gwyn"
Slide


Box="shell"
Format="l"
Msg=" How I Learned to Use Bash
====================================================

*   Total Switch from OS/2

*   Started with basic movement and list commands

*   Learned the rest as needed 

*   Starting Premise:
.   Bash can do what I want

*   Then search for the solution"
Slide


Box="shell"
Format="l"
Msg=" Why I Learned to Use Bash
====================================================


*   Fix all the stuff I broke while trying to fix the other stuff

"
Slide



################################################################################
# Normally we skip this bit
################################################################################
if [ $Skip == 0 ]
then
   Msg=" What is the Command-line interface (CLI)
   ====================================================
   
   Any text mode user interface to the Linux operating system 
   that allows the user to type commands and see the results 
   as text output."
   Slide
fi
################################################################################
# End of skipped bit
################################################################################

Box="shell"
Format="c"
Msg=" CLI Architecture
Conceptual Visualization

====================================================

+------------+    +------------+    +------------+    +------------+
|   Command  |    |            |    |  Terminal  |    |  Operating |
|   Prompt   |<==>|   Shell    |<==>|  Emulator  |<==>|   System   |
|            |    |            |    |   or VC    |    |            |
+------------+    +------------+    +------------+    +------------+"
Slide



################################################################################
# Normally we skip this bit
################################################################################
if [ $Skip == 0 ]
then
   Box="shell"
   Format="l"
   Msg=" Command Prompt
   ====================================================
   
   Command prompt - a string of characters with a cursor 
   which may be flashing - prompting you to enter a command
   
   [student@studentvm1 ~]█ "
   Slide
   
   Box="shell"
   Format="l"
   Msg=" What is a Shell?
   ====================================================
   
   *  Runs in a terminal session
   
   *  Displays the command prompt
   
   *  Interprets the commands typed by the user or SysAdmin 
   .  into a form usable by the operating system  
   
   *  Displays command results on the terminal."
   Slide
fi
################################################################################
# End of skipped bit
################################################################################


Msg=" Bash
====================================================

* Default shell for most Linux distros

* Based on Bourne shell 

* Originally developed by Stephen Bourne at Bell Labs

"
Slide

Box="stone"
Format="l"
Msg=" Some Other Shells
====================================================

* ksh
-   Written by David Korn
-   Based on Bourne shell with elements of C shell

* csh
-   C Shell for C programmers

* tcsh
-   Trusted csh
-   Mostly replaced by Bash

* zsh 
-   Contains features of csh, tcsh, and ksh"

Slide


Box=shell
Format="l"
Msg=" Old ASR 300 TTY terminal
====================================================

* An old bit of hardware that provides a means of 
. interacting with a mainframe or Unix host

* Dennis Ritchie and Ken Thompson
-   Creators of Unix

* At a PDP-11 system 
-  More powerful than the PDP-7 on which they created Unix "
Slide

# Display the terminal graphic
Pic="Terminal-01.jpg"
DispGraphic > /dev/null




################################################################################
# Normally we skip this bit
################################################################################
if [ $Skip == 0 ]
then
   Msg=" Newer Terminal
   ====================================================
   
   * DEC VT100 CRT"
   Slide
   # Display the terminal graphic
   Pic="Terminal-02.jpg"
   DispGraphic > /dev/null
fi


Msg=" Linux Virtual Console
====================================================

A means to access multiple consoles 
using a single physical system console (KVM)

Ctrl-Alt-F[1-6]"
Slide

# Display the graphic
Pic="VirtualConsole-01.jpg"
DispGraphic > /dev/null


Format="l"
Msg=" Terminal Emulator
====================================================

* A software program that emulates a hardware terminal

* Runs on GUI destop

* Many terminal emulators
-   Xterm
-   LXTerminal
-   xfce4-terminal
-   GNOME-terminal
-   Tilix
-   rxvt
-   Terminator
-   Konsole
-   ...etc."
Slide


################################################################################
# Normally we skip this bit
################################################################################
if [ $Skip == 0 ]
then
   Msg=" Pseudo-terminal 
   ====================================================
   
   *  A Linux device file to which a terminal emulator is attached
   -    Provides an interface with the operating system
   
   *  Device files for pseudo-terminals are located in /dev/pts
   
   *  The device files in /dev/pts are a number for each emulator 
   
   *  The first emulator would be /dev/pts/1, for example."
   Slide
fi
################################################################################
# End of skipped bit
################################################################################



Format="l"
Box="shell"
Msg=" Everything is a File
====================================================

Device special files - provide an open operating system 
an interface to the devices that they represent

All Linux device special files are located in the /dev directory"
Slide

Command='ls -l /dev/ | less'
DoIt


Format="l"
Box="shell"
Msg=" Or is Everything Really a Data Stream? 
====================================================

* A data stream is a file in motion

* A file is a data stream in stasis"

Slide


Box="stone"
Format="c"
Msg=" Everything is a File - Or a Data Stream
Or Whatever
====================================================

An interesting demo 

who am i

figlet -f banner 'Linux Rocks'  | boxes -d shell | lolcat > <Target>"
Slide


Msg=" Everything is a File - Or a Data Stream
Or Whatever
====================================================

Another interesting demo "
Slide

# Command differs on host david vs hosts vgr1 and vgr3
if [ $Host = "david" ]
then
   Command='sudo dd if=/dev/sda bs=2048 count=4 | less'
elif [ $Host = "voyager1" ]
then
   Command='sudo dd if=/dev/sdb bs=2048 count=8 | less '
elif [ $Host = "voyager3" ]
then
   Command='sudo dd if=/dev/nvme0n1p3 bs=2048 count=16 | less '
else
   # Best guess as to unknown host
   Command='sudo dd if=/dev/sda1 bs=2048 count=4 | less'
fi 
DoIt


Msg=" Programs and STDIO
====================================================

* Programs use STDIO for input and output rather than files 
that are stored on a disk or other recording media

* Primary function is to stream data from the output
of one program, file, or device
to the input of another program, file, or device."
Slide

Box="stone"
Format="l"

Msg=" Shell
====================================================

* Also programming languages

* Command line programming

* Can be saved as executable files"
Slide

Msg=" Shell Types 
====================================================

* Login:   A shell that results with a login
-   Direct login via virtual console
-   SSH login
-   GUI desktop is a login shell

* No Login:  A shell that is not started by a login
-   A shell started in a terminal emulator on the desktop
-   screen

* Nologin: Used by Linux system accounts
-   Specifically prevents system login
-   Used for email-only account
-   /usr/sbin/nologin "
Slide



################################################################################
################################################################################
# Beginning of Bash conifiguration file slides.                                #
################################################################################
################################################################################
Box="stone"
Format=l
Msg=" Bash Configuration Files
====================================================
* Global Login Configuration
-   /etc/bashrc
-   /etc/profile 
-   /etc/profile.d
* Global non-login Configuration
-   /etc/bash.bash_logout
* Local Login Configuration
- Takes Precedence Over Global Configuration
-   ~/.bashrc
-   ~/.bash_profile
* Other Local  Configuration
-   ~/.bash_history
-   ~/.bash_logout
* Optional - Not Usually Present With Bash
-   ~/.bash_login
-   ~/.profile"
Slide

Pic="BashConfig-01.png"
DispGraphic

Msg="File ~/.bashrc
====================================================
# .bashrc
echo \"Running ~/.bashrc\" <======== Add this line
# Source global definitions
if [ -f /etc/bashrc ]; then
    . /etc/bashrc
fi
# User specific environment
if ! [[ \$PATH =~ \$HOME/.local/bin:\$HOME/bin: ]]
then
    PATH=\$HOME/.local/bin:\$HOME/bin:\$PATH
fi
export PATH

# Uncomment the following line if you dont like 
# the systemctl auto-paging feature:
# export SYSTEMD_PAGER=
# User specific aliases and functions"
Code



Msg=" Final results with Fedora
====================================================

* Login shell
Last login: Sat Jan 18 14:40:29 2020 from ...
Running /etc/profile
Running Bash config files in /etc/profile.d
Running /etc/bashrc
Running ~/.bash_profile
Running ~/.bashrc
Running /etc/bashrc

* Non-login shell
[dboth@testvm1 ~]$ bash
Running ~/.bashrc
Running /etc/bashrc
Running Bash config files in /etc/profile.d"
Slide

Msg=" Results on POP_os! (Ubuntu)
====================================================

* Login shell
Running /etc/bash.bashrc
Running programs in /etc/profile.d
Running ~/.profile
Running ~/.bashrc
student@voyager:~$

* Non-login shell
student@voyager:~$ bash
Running /etc/bash.bashrc
Running ~/.bashrc
student@voyager:~$ "
Slide




################################################################################
# Normally we skip this bit
################################################################################
if [ $Skip == 0 ]
then
   Msg="Here Documents
   ====================================================
   
        236 ##############################################################
        237 # Display code as is with indents and all                    #
        238 ##############################################################
        239 Code()
        240 {
        241    # Clear the screen
        242    clear
        243    # Use \$Msg variable but keeps basic text formatting
        244    cat <<EOF
        245 ##############################################################
        246 \$Msg
        247 ##############################################################
        248 EOF
        249    read -r
        250 }"
   Code
fi
################################################################################
# End of skipped bit
################################################################################




Box="shell"
Format="l"
Msg=" Bash CLI Efficiency - Reduce Typing
====================================================

* Aliases

* CLI recall and editing

* Tab completion

* Expansions and Substitutions

* CLI programs"
Slide


Msg=" Aliases
====================================================

* Replace commands with longer strings

* Reduce typing

alias glances='glances -t1'
alias grep='grep --color=auto'
alias l.='ls -d .* --color=auto'
alias ll='ls -l --color=auto'
alias ls='ls --color=auto'
alias lsn='ls --color=no'
alias mc='. /usr/libexec/mc/mc-wrapper.sh'
alias vi='vim'
alias vim='vim -c \"colorscheme desert\" '"
Slide

Msg=" Command line recall and editing
====================================================

* Allows recall of previous commands
-   Can be used as-is
-   Or edited to alter the commad

* EMACS mode is default

* Can be changed to vi mode in appropriate config file
-   Local or global

* /etc/profile.d/myBashConfig.sh
"
Slide

Command='cat /etc/profile.d/myBashConfig.sh'
DoIt




Box="shell"
Format="l"
Msg=" Tab completion
====================================================

* Programmable completion facility for completing partially 
  typed program, host, file, and directory names

* Type the partial command or a file name as an argument to 
  a command, and press the Tab key

* If the host, file, directory, or program exists and the remainder 
  of the name is unique, Bash will complete the entry of the name."
Slide


Box="shell"
Format="l"
Msg="Expansions and Substitutions
====================================================

* Useful tools in programming

* Reduce typing

* Enhance shell logic

* 6 of 7 types
-   Tilde ~
-   Brace expansion {}
-   Pathname expansion *?
-   Command substitution \`\` or \$(command)
-   Arithmetic expansion
-   Parameter Substitution"
Slide

Msg="Tilde expansion (~)
====================================================

* Shortcut / alias for user home directory
-   \$HOME

ls -la ~ | less"
Slide

Command='echo $HOME'
DoIt

Command='echo ~'
DoIt




Box="shell"
Msg="Brace expansion
====================================================

* A method to use for generating arbitrary strings "
Slide

Command='echo {String1,string2,string3}'
DoIt

Command='echo b{ed,olt,ar}s'
DoIt

Command='echo testfile{0,1,2,3,4,5,6,7,8,9}.txt'
DoIt


Box="shell"
Format="l"
Command='touch {my,your,our}.test.file.{000..200}{a..f}.{txt,asc,file,text}'
DoIt

Format="c"
Msg="How Many Files Were Created?
====================================================

"
Slide

Command='ls | grep test.file | wc -l'
DoIt


Command='ls | grep test.file | less | column'
DoIt

Format="l"
Box="shell"
Msg="File and Path Name Expansion
====================================================

o Expansion of file globbing patterns using the characters ? and ∗

o Expands into the full names of files and directories which 
  match the pattern"

Command='ls our.*05?.txt | column'
DoIt


Msg="Command substitution
====================================================

* A tool that allows the STDOUT data stream of one command to be 
  used as the argument of another command

* Can generate a list of items to be processed in a loop

* Syntax
-   Backtics  \`command\` 
-   Variable \$(command)"
Slide

Command='for I in 1 2 3 4 5 6 7 8 9; do echo $I ; done'
DoIt

Command='for I in `seq 0 9` ; do echo $I ; done'
DoIt

Command='for I in $(seq 0 9) ; do echo $I ; done'
DoIt




Msg=" Arithmetic expansion
====================================================

* Substitutes the result of an arithmetic expression in a 
. program statement

* Does not require assigning the result to a variable

* Integer maths only"
Slide

Command='echo $((1+1))'
DoIt

Command='Var1=5 ; Var2=7 ; Var3=$(($Var1*$Var2)) ; echo $Var1, $Var2, $Var3'
DoIt



Msg=" Functions
====================================================

FunctionName(){list of commands ; }"
Slide

Command='hw(){ echo "Hi there kiddo" ; } ; hw'
DoIt

Box="nuke"
Format="c"
Msg=" Do NOT do this
====================================================

But what does it do?

:(){ :|:& };: "
Slide

Box="shell"
Format="l"



Box="stone"
Format="l"
Msg=" Return codes
====================================================

* All commands return a code indicating success or failure

* Return code of zero (0) indicates success

* Any other return means failure

* Some commands may return one of several non-zero codes
-   Provide more explicit information on cause of failure

* \$? variable contains the return code value
"
Slide

Command="cd /tmp ; echo \$?"
DoIt

Command="cd /root ; echo \$?"
DoIt

cd $PresentationPWD

Box="stone"
Format="l"
Msg=" Control operators
====================================================

* Use return codes to activate logical control operators

* Simple method to control flow of execution

* Easy use on CLI"
Slide

Msg=" && and ||
====================================================

* command1 && command2 
-   If command1 is successful, then run command2. 
-   If command1 fails for any reason, then command2 is skipped

* command1 || command2
-   If command1 fails, execute command2
-   If command1 succeeds, command2 is skipped

* command1 && command2 || command3
-   If command1 is successful, then run command2 but not command 3
-   If command1 fails for any reason, then skip command2 and run command3
"
Slide

Command='Dir=Testdir ; mkdir $Dir && echo "Directory $Dir created successfully." || echo "ERROR Directory $Dir not created." '
DoIt
# Clean up a bit
rm -rf $Dir
Command='Dir=/root/Testdir ; mkdir $Dir && echo "Directory $Dir created successfully." || echo "ERROR Directory $Dir not created." '
DoIt


Box="parchment"
Format="c"
Msg=" 
This is the Unix philosophy: 
====================================================
Write programs that do one thing and do it well. 
Write programs to work together. 
Write programs to handle text streams,
because that is a universal interface.

— Doug McIlroy, Basics of the Unix Philosophy"
Slide

Box="shell"
Format="l"
Msg=" What did Doug McIlroy do? "
Slide


Box="shell"
FigFont="big"
Format="l"
Msg=" Invented Pipes "
lolSlide



################################################################################
# Normally we skip this bit
################################################################################
if [ $Skip == 0 ]
then
   ################################################################################
   # A bit about philosophy
   ################################################################################
   Box="parchment"
   Format="c"
   Msg="More Unix and Linux Philosophy"
   Slide
   
   Msg="Real Sysadmins Dont sudo"
   lolSlide
   
   Msg="Bad sudo
   ====================================================
   
   * Some distros like Ubuntu force users to use sudo
   -   sudo <command> then enter password
   
   * Extra keystrokes means slower and less efficient
   -   Lazy Admin always looking for ways to reduce typing
   
   * Users can make same errors
   -   It just takes longer
   
   * Much less secure
   -   IN MY OPINION! "
   Slide
   
   Msg=" Good sudo
   ====================================================
   
   * Used to allow limited access to programs that require root priviledge
   -   Limit users
   -   Limit programs
   
   *  sudo dd if=/dev/sda bs=2048 count=8
   "
   Slide

fi

Box="girl"
Format="c"
Msg=" Linux Philosophy for SysAdmins
====================================================


############  Automate everything  ############"
Slide


clear
Msg="A SysAdmin is most productive when thinking."
echo $Msg | cowsay -W 65
read -r

Box="c"
Format="l"
Msg=" Thinking about ...
====================================================

* How to solve existing problems 
* How to avoid future problems
* How to monitor Linux computers 
-   Find clues that anticipate future problems
* How to perform our job more efficiently
* How to automate all of those tasks that need to be performed
-   Whether every day or once a year."
Slide

clear
Msg="  ... But what will the PHB think???" 
echo $Msg | cowthink  -p 
read -r


clear
Msg=" SysAdmins are next most productive when creating the shell programs that automate the solutions that they have conceived while appearing to be unproductive."
echo $Msg | cowsay -W 65 
read -r


################################################################################
# PHB stories
################################################################################
Box="shell"
Format="c"
Msg="PHB Encounters of the Third Kind

====================================================

* The LBK

* The List
"
Slide

Box="shell"
Format="c"
Msg="PHB Encounters of the Third Kind

====================================================

* The Solution

"
Slide

Command="rpm -qa | wc"
DoIt


Command='rpm -qa'
DoIt

Command='rpm -qa | sort | head -50' 
DoIt

Command='for Y in $(rpm -qa | sort | head -50) ; do echo ;  echo "Name : $Y" ; done'
DoIt

Command='for Y in $(rpm -qa | sort | head -50) ; do echo ;  echo "Name : $Y" ; rpm -qi $Y | grep Summary ; done'
DoIt

Box="stone"
Format="l"
Msg="More Linux Philosophy
====================================================
* Make Programs Portable
-  Shell scripts are perfect
-  Z390 test

* Test early and test often
-   Fuzzy Testing
-   Test in production

* Strive for Elegance

* Find the Simplicity

* Follow your curiosity
"
Slide


   
################################################################################
# About Licensing
################################################################################
Box="columns"
Format="c"
FigFont="big"
Msg="Licensing Your Work"
lolSlide

Box="shell"
Format="l"
Msg="About Licensing for SysAdmins
====================================================
* SysAdmins Write lots of code for automation

* We also like to share our code

* License your code 

* Prevents preception of legal liabilities

* Company lawyers need a license

* Makes software Free as in Speech

* Keeps it Free as in Beer
"
Slide

   
Box="shell"
Format="c"
FigFont=term
Msg="It's Not Open Source
Until
It Has A License"
lolSlide


Box="stone"
Format="l"
Command="less demo.sh"
DoIt

################################################################################
################################################################################
################################################################################
################################################################################
#
#

Box="shell"
Format="c"
FigFont=banner
Msg="Automate Everything"
lolSlide

Msg=" Additional Resources
====================================================

The Art of Unix Programming -- Eric Steven Raymond
http://www.catb.org/~esr/writings/taoup/html/

Linux and the Unix Philosopy -- Mike Gancarz
https://flylib.com/books/en/2.506.1/

The source code is the license  -- Scott K Peterson
https://opensource.com/article/17/12/source-code-license

Advanced Bash-Scripting Guide -- Mendel Cooper
https://tldp.org/LDP/abs/html/

13 resources for learning to write better Bash code -- Jose Vicente Nunez
https://www.redhat.com/sysadmin/learn-bash-scripting

"
Slide

Box="shell"
Format="l"
Msg="Bash man page
====================================================

* Bash command syntax
* List of shell variables
* Programming tools
-   Flow control
-   Functions
* Option arguments
"
Slide

################################################################################
# Do some cleanup
################################################################################
rm *test.file* ; rm -rf Testdir


Box="shell"
Format="c"
FigFont=banner
Msg="Please Rate My Presentation"
lolSlide


Box="shell"
Format="c"
FigFont=banner
Msg="Thank You!"
lolSlide

# The steam locomotive
#sl

# The final fun thing
Command='asciiquarium'
DoIt
Quit

################################################################################
################################################################################
################################################################################
################################################################################
# End of program
################################################################################
################################################################################
################################################################################
################################################################################


